"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@code-to-json/core");
const fs = require("fs");
const glob = require("glob");
const path = require("path");
const ts = require("typescript");
const util_1 = require("util");
const __1 = require("..");
const invalid_arguments_error_1 = require("../invalid-arguments-error");
const DEFAULT_COMPILER_OPTIONS = {
    allowJs: true,
    checkJs: true,
    moduleResolution: ts.ModuleResolutionKind.NodeJs,
    target: ts.ScriptTarget.ESNext
};
function tsConfigForPath(filePath) {
    return ts.findConfigFile(filePath, (fileName) => true);
}
const pGlob = util_1.promisify(glob);
async function globsToPaths(globs, extensions = ['.js', '.ts']) {
    const valueSet = new Set();
    await Promise.all(globs.map(g => pGlob(g)
        .then((files) => {
        files.forEach(f => valueSet.add(f));
    })
        .catch(er => {
        throw new invalid_arguments_error_1.default(`Invalid glob: ${g}\n${er}`);
    })));
    const allPaths = [...valueSet];
    // If extensions are provided, only return those files that match
    return extensions
        ? allPaths.filter(f => extensions.indexOf(path.extname(f).toLowerCase()) >= 0)
        : allPaths;
}
async function createProgramFromTsConfig(searchPath) {
    const cfgPath = tsConfigForPath(searchPath);
    if (!cfgPath) {
        throw new invalid_arguments_error_1.default(`Could not find a tsconfig.json via path "${searchPath}"`);
    }
    __1.debugLog('Found typescript configuration file: ', cfgPath);
    const configContent = ts.readConfigFile(cfgPath, (filePath) => fs.readFileSync(filePath).toString());
    const { error, config } = configContent;
    if (error) {
        __1.debugLog('tsconfig diagnostics', error);
        throw new invalid_arguments_error_1.default('TSConfig error - ' + error.messageText);
    }
    else if (config) {
        const diagReporter = ts.createDiagnosticReporter(ts.sys);
        const configResult = ts.getParsedCommandLineOfConfigFile(cfgPath, {}, ts.sys);
        if (!configResult) {
            throw new invalid_arguments_error_1.default(`Failed to parse config file "${cfgPath}"`);
        }
        const { fileNames: rootNames, options, errors } = configResult;
        if (errors && errors.length > 0) {
            __1.debugLog('tsconfig config parse errors', errors);
            throw new invalid_arguments_error_1.default(`Detected errors while parsing tsconfig file: ${JSON.stringify(errors)}`);
        }
        __1.debugLog('Using typescript compiler options', options);
        __1.debugLog('applying to files', rootNames);
        return ts.createProgram({
            rootNames,
            options
        });
    }
    else {
        throw new invalid_arguments_error_1.default('reading tsconfig seemed to neither suceed or fail');
    }
}
async function createProgramFromEntries(globs) {
    const rootNames = await globsToPaths(globs);
    __1.debugLog('Globs are equivalent to files', rootNames);
    const prog = ts.createProgram({
        rootNames,
        options: DEFAULT_COMPILER_OPTIONS
    });
    return prog;
}
async function run(options, rawEntries) {
    const { project, out = 'out.json' } = options;
    let program;
    if (typeof project === 'string') {
        program = await createProgramFromTsConfig(project);
    }
    else if (!project && rawEntries && rawEntries.length > 0) {
        program = await createProgramFromEntries(rawEntries);
    }
    else {
        throw new invalid_arguments_error_1.default('Either --program <path> or entries glob(s) must be defined');
    }
    const walkResult = core_1.walkProgram(program);
    // debugLog('walk result', walkResult);
    // console.log(walkResult);
    const outputPath = path.isAbsolute(out) ? out : path.join(process.cwd(), out);
    fs.writeFileSync(outputPath, JSON.stringify(walkResult, null, '  '));
}
exports.default = run;
//# sourceMappingURL=run.js.map